# Blind Proxy

A secure proxy server for the Blind Insight API that provides local encryption/decryption and enhanced functionality.

## Features

### API Proxying

- Secure proxying of API requests to the Blind Insight server
- Local encryption of sensitive data before transmission
- Local decryption of received data
- CORS support with configurable allowed origins

### Batch Processing and Concurrency

The proxy implements an efficient parallel processing system for handling large volumes of records:

- **Batch Processing**: Records are automatically processed in configurable batches

  - Default batch size: 10 records
  - Maximum batch size: 50 records
  - Customizable batch size within limits
  - Configurable sleep interval between batches (default: 100ms)
  - Rate limiting to prevent overwhelming the server

- **Parallel Processing**:

  - Configurable number of concurrent workers
  - Default worker count optimized for CPU cores (2n + 1)
  - Automatic load balancing across workers

- **Real-time Progress Updates**:

  - WebSocket support for live progress monitoring
  - Progress information includes:
    - Current status (processing/complete/failed)
    - Number of records processed
    - Current batch number
    - Total number of batches
    - Error information (if any)

- **Job Management**:
  - Unique job IDs for tracking uploads
  - Job status persistence for 1 hour after completion
  - Support for multiple WebSocket clients monitoring the same job
  - Automatic cleanup of completed jobs

### File Upload Support

- TUS protocol support for resumable file uploads
- Configurable upload directory
- Progress tracking for file uploads

## API Endpoints

### Bulk Upload Endpoints

```no-highlight
POST /api/jobs/upload/           # Start a new bulk upload job
GET  /api/jobs/:job_id/         # Get job status
GET  /api/ws/jobs/:job_id/      # WebSocket endpoint for real-time updates
```

### File Upload Endpoints

```no-highlight
POST  /api/files/               # Start a new file upload
HEAD  /api/files/:id           # Get upload metadata
PATCH /api/files/:id           # Upload file chunks
GET   /api/files/:id           # Download uploaded file
```

### Record Management

```no-highlight
POST /api/records/ingest/      # Ingest records with local encryption
POST /api/records/search/      # Search records with local decryption
POST /api/records/decrypt/     # Decrypt records locally
```

### Schema Management

```no-highlight
POST /api/schemas/decrypt/     # Decrypt schema fields locally
```

## Usage Examples

### Starting a Bulk Upload

Request:

```http
POST /api/jobs/upload/
{
    "records": [
        {
            "schemaUrl": "/api/schemas/person/",
            "data": {
                "name": "John Doe",
                "age": 30
            }
        }
        // ... more records
    ],
    "batchSize": 10  // Optional, defaults to 10
}
```

Response:

```json
{
  "job_id": "550e8400-e29b-41d4-a716-446655440000"
}
```

### Monitoring Upload Progress

Get job status:

```http
GET /api/jobs/:job_id/
```

Connect to WebSocket for real-time updates:

```http
GET /api/ws/jobs/:job_id/
```

WebSocket messages will contain progress updates in the following format:

```json
{
  "status": "processing",
  "processed": 50,
  "total": 100,
  "current_batch": 5,
  "total_batches": 10
}
```

## Configuration

The proxy can be configured with the following options:

- `Server`: Target Blind Insight API server URL
- `InsecureSkipVerify`: Whether to skip SSL verification
- `AllowedOrigins`: List of allowed CORS origins
- `DefaultWallet`: Default wallet for API requests
- `WorkerCount`: Number of concurrent workers for parallel processing (defaults to 2n + 1, where n is CPU cores)
- `BatchSleepInterval`: Time to sleep between processing batches (defaults to 100ms)

## Running the Proxy as a Server

You can start the proxy as a server by running:

```bash
blind proxy
```

When debugging or developing:

```bash
blind proxy --debug
```

IF you want ALL THE LOGS enable trace logging with `DEBUG=trace`:

```bash
DEBUG=trace blind proxy --debug
```

## Development

### Quick Setup

To quickly get your dev environment ready for local builds run:

```bash
script/setup
```

### Getting the Certs

To run the proxy locally or do local builds you'll need the `local.blindinsight.io` certificates to be available.

[!NOTE]
You will need the 1Password CLI `op` command to use this script. # If you don't have the 1Password CLI installed, install it: [https://developer.1password.com/docs/cli/install](https://developer.1password.com/docs/cli/install)

Easy mode:

```bash
script/getcerts
```

Hard mode (doesn't require `op` command):

```bash
$ script/certs
Target directory for certs: /Users/jathan/sandbox/src/proxy/config/pets
Usage: script/certs <fullchain> <privkey>
```

Copy fullchain.pem from 1Password and then:

```bash
export CERT=$(pbpaste)
```

Copy privkey.pem from 1Password and then:

```bash
export KEY=$(pbpaste)
```

And then:

```bash
script/certs $CERT $KEY
```

### Downloading the Web assets

Additionally, you'll also need the web assets to be able to point to `https://local.blindinsight.io` in your browser.

You will need to use a [GitHub Personal Access Token](https://github.com/settings/tokens) and then set it as `GITHUB_TOKEN` environment variable.

```bash
export GITHUB_TOKEN={your_token}
```

### Running the Server from source

```bash
go run cmd/blind/blind.go --proxy
```
